import schedule
import threading
import time
from xt_trader import XtTrader
from flask import Flask, request, abort, jsonify
from config import Config
from concurrent.futures import TimeoutError
import concurrent.futures
import atexit
from simulation_trader import SimulationTrader
import datetime
from strategy_position_manager import StrategyPositionManager
from logger_config import get_logger

# 获取日志记录器
logger = get_logger('server')

# 全局交易实例（采用单例模式）
_sim_trader_instance = None  # 模拟交易实例（单例）
_real_trader_instance = None  # 实盘交易实例（单例）

# 获取模拟交易实例的辅助函数
def get_sim_trader():
    """获取模拟交易实例 - 保证单例模式
    
    Returns:
        返回模拟交易单例实例
    """
    global _sim_trader_instance
    if _sim_trader_instance is None:
        _sim_trader_instance = SimulationTrader()
    return _sim_trader_instance

# 获取实盘交易实例的辅助函数
def get_real_trader():
    """获取实盘交易实例 - 保证单例模式
    
    Returns:
        返回实盘交易单例实例
    """
    global _real_trader_instance
    if _real_trader_instance is None:
        _real_trader_instance = XtTrader()
        # 检查交易实例是否已登录，如果未登录则进行登录
        if not _real_trader_instance.is_logged_in():
            logger.info("创建新的XtTrader实例并登录")
            _real_trader_instance.login()
    return _real_trader_instance

# 获取交易实例
def get_trader(use_sim_trader=False):
    """获取交易实例 - 采用单例模式
    
    Args:
        use_sim_trader (bool): 是否强制使用模拟交易，True表示必定返回模拟交易实例
        
    Returns:
        返回交易实例，根据参数和配置决定是模拟交易还是实盘交易
    """
    # 如果强制使用模拟交易，返回模拟交易单例
    if use_sim_trader:
        return get_sim_trader()
    
    # 如果配置为仅模拟交易，返回模拟交易单例
    if Config.SIMULATION_ONLY:
        return get_sim_trader()
    
    # 判断当前是否为交易时间
    now = datetime.datetime.now()
    current_time = now.time()
    
    # 是否在交易时间段内（9:30-11:30, 13:00-15:00）
    morning_start = datetime.time(9, 30)
    morning_end = datetime.time(11, 30)
    afternoon_start = datetime.time(13, 0)
    afternoon_end = datetime.time(15, 0)
    
    is_trading_hour = (morning_start <= current_time <= morning_end) or (afternoon_start <= current_time <= afternoon_end)
    
    # 尝试导入chinese_calendar判断是否为交易日
    try:
        from chinese_calendar import is_workday, is_holiday
        is_trading_day = is_workday(now) and not is_holiday(now)
    except ImportError:
        # 如果无法导入chinese_calendar，则简单地用工作日判断
        is_trading_day = now.weekday() < 5  # 0-4 为周一至周五
    
    # 如果不是交易日或不在交易时间内，返回模拟交易单例
    if not is_trading_day or not is_trading_hour:
        logger.info(f"当前非交易时段 - 日期: {now.date()}, 时间: {current_time}, 使用模拟交易")
        return get_sim_trader()
    
    # 否则返回真实交易单例
    return get_real_trader()

def run_daily(time_str, job_func):
    """设置每天在指定时间运行的任务
    
    Args:
        time_str: 运行时间，格式为"HH:MM"
        job_func: 要运行的函数
    """
    # 不再区分周一到周五，而是每天执行
    # 交易日判断逻辑已移到get_trader函数中
    schedule.every().day.at(time_str).do(job_func)


def run_pending_tasks():
    while True:
        try:
            schedule.run_pending()
            time.sleep(1)
        except Exception as e:
            logger.error(f"Error in scheduler: {str(e)}")


# Run the task scheduler in a new thread
threading.Thread(target=run_pending_tasks).start()

# 初始化交易环境
get_trader().login()

# 添加请求频率限制
app = Flask(__name__)

# 添加策略数据相关的定期任务
schedule.every().day.at("00:01").do(StrategyPositionManager.clean_timeout_orders)  # 每天清理超时委托
schedule.every().day.at("15:30").do(StrategyPositionManager.save_strategy_data)  # 每天收盘后保存策略数据

# 程序启动时加载策略数据
StrategyPositionManager.load_strategy_data()

# 程序退出时保存策略数据
atexit.register(StrategyPositionManager.save_strategy_data)

# 使用配置文件中的时间
run_daily(Config.MARKET_OPEN_TIME, lambda: get_trader().login())
run_daily(Config.MARKET_ACTIVE_TIME, lambda: get_trader().get_balance())
run_daily(Config.MARKET_CLOSE_TIME, lambda: get_trader().logout())


@app.route("/yu/healthcheck", methods=["GET"])
def health_check():
    return "ok", 200


def should_use_simulation():
    """判断是否应该使用模拟交易
    
    Returns:
        tuple: (should_simulate: bool, simulation_reason: str)
        should_simulate: 是否应该使用模拟交易
        simulation_reason: 使用模拟交易的原因
    """
    # 直接使用get_trader()返回的实例类型判断
    trader = get_trader()
    
    if isinstance(trader, SimulationTrader):
        # 获取原因
        if Config.SIMULATION_ONLY:
            return True, "配置为仅模拟交易"
        else:
            now = datetime.datetime.now()
            return True, f"当前非交易时段 - {now.strftime('%Y-%m-%d %H:%M:%S')}"
    
    # 如果是实盘交易实例
    return False, ""


@app.route("/yu/buy", methods=["POST"])
def buy():
    """Buy an item with given parameters."""
    logger.info("Received buy request")
    # 每次操作前更新未完成委托状态
    current_trader = get_trader()
    StrategyPositionManager.update_pending_orders(current_trader)
    
    # Get data from request body
    data = request.get_json()
    code = data.get("code")
    price_str = data.get("price")
    amount_str = data.get("amount")
    strategy_name = data.get("strategy_name", "")  # 新增策略名称参数，默认为空

    try:
        if not all([code, price_str, amount_str]):
            raise ValueError("Missing required parameters")
        
        price = float(price_str)
        amount = int(amount_str)
        
        if price <= 0 or amount <= 0:
            raise ValueError("Price and amount must be positive")
        
        # 检查是否需要模拟交易
        should_simulate, simulation_reason = should_use_simulation()
        
        if should_simulate:
            # 使用模拟交易
            logger.info(f"使用模拟交易 - {simulation_reason} - 代码: {code}, 价格: {price}, 数量: {amount}")
            
            # 获取模拟交易实例并执行买入操作
            sim_trader = get_trader(True)
            result = sim_trader.buy(code, price, amount)
            
            # 如果指定了策略名称，记录到策略持仓
            if strategy_name:
                # 模拟交易立即生效，更新策略持仓
                StrategyPositionManager.update_strategy_position(sim_trader, strategy_name, code, 'buy', amount)
                
            return jsonify({"success": True, "data": result}), 200
            
        # 尝试实盘交易
        logger.info(f"Executing buy order: code={code}, price={price}, amount={amount}, strategy_name={strategy_name}")
        try:
            result = execute_with_timeout(current_trader.buy, Config.TRADE_TIMEOUT, code, price, amount)
            if result is None:
                # 超时时使用模拟交易
                logger.warning(f"Buy order timeout after {Config.TRADE_TIMEOUT} seconds, switching to simulation mode")
                
                # 创建模拟交易实例并执行买入操作
                sim_trader = get_trader(True)
                result = sim_trader.buy(code, price, amount)
                
                # 如果指定了策略名称，记录到策略持仓
                if strategy_name:
                    # 超时情况下，使用模拟交易，立即更新策略持仓
                    StrategyPositionManager.update_strategy_position(sim_trader, strategy_name, code, 'buy', amount)
                    
                return jsonify({"success": True, "data": result}), 200
            
            # 如果指定了策略名称且是真实交易
            if strategy_name and 'order_id' in result and result['order_id'] != 'simulation':
                order_id = result['order_id']
                
                # 添加到未完成委托
                StrategyPositionManager.add_pending_order(
                    current_trader,
                    order_id,
                    strategy_name,
                    code,
                    price,
                    amount,
                    'buy'
                )
                
                # 注意：不在这里调用update_strategy_position
                # 持仓更新将由update_pending_orders函数处理
                # 这避免了持仓更新的冗余操作
            
            logger.info(f"Buy order result: {result}")
            return jsonify({"success": True, "data": result}), 200
        except Exception as e:
            # 发生错误时使用模拟交易
            logger.error(f"Buy order failed: {str(e)}, switching to simulation mode")
            
            # 创建模拟交易实例并执行买入操作
            sim_trader = get_trader(True)
            result = sim_trader.buy(code, price, amount)
            
            # 如果指定了策略名称，记录到策略持仓
            if strategy_name:
                # 错误情况下，使用模拟交易，立即更新策略持仓
                StrategyPositionManager.update_strategy_position(sim_trader, strategy_name, code, 'buy', amount)
                
            return jsonify({"success": True, "data": result}), 200
    except ValueError as e:
        logger.error(f"Invalid request parameters: {str(e)}")
        abort(400, description=str(e))
    except Exception as e:
        logger.error(f"Error processing buy request: {str(e)}")
        abort(500, description="Internal server error")


@app.route("/yu/sell", methods=["POST"])
def sell():
    """Sell an item with given parameters."""
    logger.info("Received sell request")
    # 每次操作前更新未完成委托状态
    current_trader = get_trader()
    StrategyPositionManager.update_pending_orders(current_trader)
    
    # Get data from request body
    data = request.get_json()
    code = data.get("code")
    price_str = data.get("price")
    amount_str = data.get("amount")
    strategy_name = data.get("strategy_name", "")  # 新增策略名称参数，默认为空

    try:
        if not all([code, price_str, amount_str]):
            raise ValueError("Missing required parameters")
        
        price = float(price_str)
        amount = int(amount_str)
        
        if price <= 0 or amount <= 0:
            raise ValueError("Price and amount must be positive")
        
        # 检查是否需要模拟交易
        should_simulate, simulation_reason = should_use_simulation()
        
        if should_simulate:
            # 使用模拟交易
            logger.info(f"使用模拟交易 - {simulation_reason} - 代码: {code}, 价格: {price}, 数量: {amount}")
            
            # 获取模拟交易实例并执行卖出操作
            sim_trader = get_trader(True)
            result = sim_trader.sell(code, price, amount)
            
            # 如果指定了策略名称，记录到策略持仓
            if strategy_name:
                # 模拟交易下，使用简单更新模式
                StrategyPositionManager.update_strategy_position(sim_trader, strategy_name, code, 'sell', amount)
                
            return jsonify({"success": True, "data": result}), 200
            
        # 尝试实盘交易
        logger.info(f"Executing sell order: code={code}, price={price}, amount={amount}, strategy_name={strategy_name}")
        try:
            result = execute_with_timeout(current_trader.sell, Config.TRADE_TIMEOUT, code, price, amount)
            if result is None:
                # 超时时使用模拟交易
                logger.warning(f"Sell order timeout after {Config.TRADE_TIMEOUT} seconds, switching to simulation mode")
                
                # 创建模拟交易实例并执行卖出操作
                sim_trader = get_trader(True)
                result = sim_trader.sell(code, price, amount)
                
                # 如果指定了策略名称，记录到策略持仓
                if strategy_name:
                    # 超时情况下，使用简单更新模式
                    StrategyPositionManager.update_strategy_position(sim_trader, strategy_name, code, 'sell', amount)
                    
                return jsonify({"success": True, "data": result}), 200
            
            # 如果指定了策略名称，记录到未完成委托
            if strategy_name and 'order_id' in result and result['order_id'] != 'simulation':
                order_id = result['order_id']
                
                # 添加到未完成委托
                StrategyPositionManager.add_pending_order(
                    current_trader,
                    order_id,
                    strategy_name,
                    code,
                    price,
                    amount,
                    'sell'
                )
            
            logger.info(f"Sell order result: {result}")
            return jsonify({"success": True, "data": result}), 200
        except Exception as e:
            # 发生错误时使用模拟交易
            logger.error(f"Sell order failed: {str(e)}, switching to simulation mode")
            
            # 创建模拟交易实例并执行卖出操作
            sim_trader = get_trader(True)
            result = sim_trader.sell(code, price, amount)
            
            # 如果指定了策略名称，记录到策略持仓
            if strategy_name:
                # 错误情况下，使用简单更新模式
                StrategyPositionManager.update_strategy_position(sim_trader, strategy_name, code, 'sell', amount)
                
            return jsonify({"success": True, "data": result}), 200
    except ValueError as e:
        logger.error(f"Invalid request parameters: {str(e)}")
        abort(400, description=str(e))
    except Exception as e:
        logger.error(f"Error processing sell request: {str(e)}")
        abort(500, description="Internal server error")


@app.route("/yu/cancel/<entrust_no>", methods=["DELETE"])
def cancel(entrust_no):
    logger.info(f"Received cancel request for entrust_no={entrust_no}")
    try:
        current_trader = get_trader()
        result = current_trader.cancel(entrust_no)
        logger.info(f"Cancel result: {result}")
        
        # 更新未完成委托状态
        StrategyPositionManager.update_pending_orders(current_trader)
        
        response = {"success": True, "data": result}
        return jsonify(response), 200
    except Exception as e:
        logger.error(f"Error processing cancel request: {str(e)}")
        abort(500, description="Internal server error")


@app.route("/yu/balance", methods=["GET"])
def get_balance():
    """Get the balance of the account."""
    logger.info("Received balance request")
    try:
        current_trader = get_trader()
        balance = current_trader.get_balance()
        logger.info(f"Balance: {balance}")

        response = {"success": True, "data": balance}
        return jsonify(response), 200
    except Exception as e:
        print(e)
        logger.error(f"Error processing balance request: {str(e)}")
        abort(500, description="Internal server error")


@app.route("/yu/positions", methods=["GET"])
def get_positions():
    """Get the positions of the account."""
    logger.info("Received positions request")
    # 每次查询前更新未完成委托状态
    current_trader = get_trader()
    StrategyPositionManager.update_pending_orders(current_trader)
    
    try:
        # 获取查询参数中的策略名称
        strategy_name = request.args.get("strategy_name", "")
        
        # 使用StrategyPositionManager获取持仓信息
        result = StrategyPositionManager.get_strategy_positions(current_trader, strategy_name if strategy_name else None)
        
        return jsonify({"success": True, "data": result}), 200
    except Exception as e:
        logger.error(f"Error processing positions request: {str(e)}")
        abort(500, description="Internal server error")


@app.route("/yu/todaytrades", methods=["GET"])
def get_today_trades():
    """Get the today's trades of the account."""
    logger.info("Received today trades request")
    try:
        current_trader = get_trader()
        trades = current_trader.get_today_trades()
        logger.info(f"Today trades: {trades}")

        response = {"success": True, "data": trades}
        return jsonify(response), 200
    except Exception as e:
        logger.error(f"Error processing today trades request: {str(e)}")
        abort(500, description="Internal server error")


@app.route("/yu/todayentrust", methods=["GET"])
def get_today_entrust():
    """Get the today's entrust of the account."""
    logger.info("Received today entrust request")
    try:
        current_trader = get_trader()
        entrust = current_trader.get_today_entrust()
        logger.info(f"Today entrust: {entrust}")

        response = {"success": True, "data": entrust}
        return jsonify(response), 200
    except Exception as e:
        logger.error(f"Error processing today entrust request: {str(e)}")
        abort(500, description="Internal server error")


@app.route("/yu/clear/<strategy_name>", methods=["DELETE"])
def clear_strategy(strategy_name):
    """清除指定策略的持仓管理数据"""
    logger.info(f"接收到清除策略持仓请求: {strategy_name}")
    try:
        current_trader = get_trader()
        
        # 如果是模拟交易实例，则重置模拟交易实例
        if isinstance(current_trader, SimulationTrader):
            global _sim_trader_instance
            if _sim_trader_instance is not None:
                logger.info("重置模拟交易实例")
                # 创建一个新的模拟交易实例，替换原有实例
                _sim_trader_instance = SimulationTrader()
                current_trader = _sim_trader_instance
        
        # 使用StrategyPositionManager清除策略
        success, message = StrategyPositionManager.clear_strategy(current_trader, strategy_name)
        
        if success:
            return jsonify({"success": True, "message": message}), 200
        else:
            abort(400, description=message)
            
    except Exception as e:
        logger.error(f"清除策略持仓时出错: {str(e)}")
        abort(500, description="服务器内部错误")


# 超时处理函数
def execute_with_timeout(func, timeout, *args, **kwargs):
    with concurrent.futures.ThreadPoolExecutor(max_workers=1) as executor:
        future = executor.submit(func, *args, **kwargs)
        try:
            return future.result(timeout=timeout)
        except TimeoutError:
            return None


if __name__ == "__main__":
    logger.info(f"Server starting on {Config.HOST}:{Config.PORT}")
    app.run(debug=Config.DEBUG, host=Config.HOST, port=Config.PORT)
